/*
 * @(#)PDFGraphicsDemo.java	Version 1.0  <April 02, 2003>
 *
 * Copyright 2000-2002 Elegant MicroWeb Technologies Pvt. Ltd. (EMTPL). All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * - Redistributions of source code is not permitted under any conditions.
 * 
 * - Redistribution in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * 
 * Neither the name of Elegant MicroWeb Technologies Pvt. Ltd. or the names of contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 * 
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES,
 * INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. EMTPL AND
 * ITS LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES OR LIABILITIES
 * SUFFERED BY LICENSEE AS A RESULT OF  OR RELATING TO USE, MODIFICATION
 * OR DISTRIBUTION OF THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL
 * EMTPL OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE
 * DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,
 * ARISING OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF EMTPL HAS
 * BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * 
 * You acknowledge that Software is not designed, licensed or intended
 * for use in the design, construction, operation or maintenance of any
 * nuclear facility or any other illegal activity.
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import java.beans.*;
import java.util.*;
import java.io.*;

import common.*;
import com.elegantj.pdf.*;

public class PDFGraphicsDemo extends JFrame
{
	JPanel mainPanel;
	JTabbedPane tabPane;

	JFCPanelAppPanel jfcPanelAppPanel;
	JFCTextAreaAppPanel jfcTextAreaAppPanel;
	JFCTextPaneAppPanel jfcTextPaneAppPanel;
	JFCTableAppPanel jfcTableAppPanel;

	ButtonsAppPanel buttonsAppPanel;
	
	PropertyEditor colorEditor;
	PropertyEditor fontEditor;

	public PDFGraphicsDemo()
	{
		super();
		
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		setTitle("ElegantJ PDF Library Demo - PDFGraphics Sample");
		
		javax.swing.plaf.metal.MetalLookAndFeel.setCurrentTheme(new PDFTheme());
		try
		{
  		UIManager.setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel");
  		SwingUtilities.updateComponentTreeUI(this);
		}
		catch (Exception ex)
		{
		}		
		
		colorEditor = PropertyEditorManager.findEditor(java.awt.Color.class);
  	fontEditor = PropertyEditorManager.findEditor(java.awt.Font.class);		

		mainPanel = new JPanel();
		mainPanel.setLayout(new BorderLayout(5, 5));

		tabPane = new JTabbedPane();

		jfcTextAreaAppPanel = new JFCTextAreaAppPanel(this);
		jfcTextPaneAppPanel = new JFCTextPaneAppPanel(this);
		jfcTableAppPanel = new JFCTableAppPanel(this);
		jfcPanelAppPanel = new JFCPanelAppPanel(this);

		tabPane.addTab("JTextArea", jfcTextAreaAppPanel);
		tabPane.addTab("JTextPane", jfcTextPaneAppPanel);
		tabPane.addTab("JTable", jfcTableAppPanel);
		tabPane.addTab("JPanel", jfcPanelAppPanel);

		buttonsAppPanel = new ButtonsAppPanel(this);

		mainPanel.add(tabPane, BorderLayout.CENTER);
		mainPanel.add(buttonsAppPanel, BorderLayout.SOUTH);

		getContentPane().add(mainPanel, BorderLayout.CENTER);
	}

	public static void main(String args[])
	{
		PDFGraphicsDemo pdfGraphicsDemo = new PDFGraphicsDemo();
		Dimension dim = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
		int preferredWidth = dim.width - ((dim.width * 10)/100);
		int preferredHeight = dim.height - ((dim.height * 10)/100);
		pdfGraphicsDemo.setBounds((dim.width - preferredWidth)/2, (dim.height - preferredHeight)/2, preferredWidth, preferredHeight);
		pdfGraphicsDemo.show();
	}
}

class JFCPanelAppPanel extends JPanel
{
	PDFGraphicsDemo pdfGraphicsDemo;

	JPanel panel;
	
	JLabel label1;
	JLabel label2;
	JLabel label3;
	JLabel label4;
	JLabel label5;

	JTextField textField1;
	JTextField textField2;
	JTextField textField3;
	JTextField textField4;
	JTextField textField5;

	public JFCPanelAppPanel(PDFGraphicsDemo pdfGraphicsDemo)
	{
		this.pdfGraphicsDemo = pdfGraphicsDemo;

		setLayout(new FlowLayout(FlowLayout.CENTER));

		textField1 = new JTextField(30);
		textField2 = new JTextField(30);
		textField3 = new JTextField(30);
		textField4 = new JTextField(30);
		textField5 = new JTextField(30);

		label1 = new JLabel("Sr No. : ");
		label2 = new JLabel("Name : ");
		label3 = new JLabel("Address : ");
		label4 = new JLabel("Postal Code : ");
		label5 = new JLabel("Email : ");

		reset();

		panel = new JPanel()
		{
			public Dimension getPreferredSize()
			{
				return new Dimension(700, 200);
			}
		};
		panel.setBorder(BorderFactory.createTitledBorder(
			BorderFactory.createCompoundBorder(BorderFactory.createLineBorder(Color.black), 
				BorderFactory.createEmptyBorder(5, 5, 5, 5)), " Enter Information for Contact .", 
				javax.swing.border.TitledBorder.LEFT, javax.swing.border.TitledBorder.TOP, 
				new Font("Dialog", Font.BOLD, 12), Color.black));
		panel.setLayout(new BorderLayout(5, 5));
		
		JPanel leftPanel = new JPanel();
		leftPanel.setLayout(new GridLayout(5, 1, 10, 5));
		
		JPanel rightPanel = new JPanel();
		rightPanel.setLayout(new GridLayout(5, 1, 10, 5));
		
		leftPanel.add(label1);	rightPanel.add(textField1);
		leftPanel.add(label2);	rightPanel.add(textField2);
		leftPanel.add(label3);	rightPanel.add(textField3);
		leftPanel.add(label4);	rightPanel.add(textField4);
		leftPanel.add(label5);	rightPanel.add(textField5);
		
		panel.add(leftPanel, BorderLayout.WEST);
		panel.add(rightPanel, BorderLayout.CENTER);

		add(panel);
	}

	public JPanel getPanel()
	{
		return panel;
	}

	public Insets getInsets()
	{
		return new Insets(10, 20, 10, 20);
	}

	public void reset()
	{
		textField1.setText("");
		textField2.setText("");
		textField3.setText("");
		textField4.setText("");
		textField5.setText("");
	}
}

class JFCTextAreaAppPanel extends JPanel
{
	PDFGraphicsDemo pdfGraphicsDemo;

	JTextArea textArea;

	JPanel leftPanel;
	RightAppPanel rightAppPanel;

	public JFCTextAreaAppPanel(PDFGraphicsDemo pdfGraphicsDemo)
	{
		this.pdfGraphicsDemo = pdfGraphicsDemo;
		setLayout(new BorderLayout(5, 5));

		textArea = new JTextArea();
		reset();

		rightAppPanel = new RightAppPanel(pdfGraphicsDemo, textArea);

		leftPanel = new JPanel();
		leftPanel.setLayout(new BorderLayout(5, 5));
		leftPanel.add(new JScrollPane(textArea), BorderLayout.CENTER);

		add(leftPanel, BorderLayout.CENTER);
		add(rightAppPanel, BorderLayout.EAST);
	}

	public JTextArea getTextArea()
	{
		return textArea;
	}

	public void reset()
	{
		textArea.setText("This is a JFC TextArea\nTo view page setup click PageSetup\nTo view preview click Preview\nTo take print click Print");
		textArea.setFont(new Font("Dialog", Font.PLAIN, 12));
		textArea.setForeground(Color.black);
	}
}

class JFCTextPaneAppPanel extends JPanel
{
	PDFGraphicsDemo pdfGraphicsDemo;

	JTextPane textPane;

	JPanel leftPanel;
	RightAppPanel rightAppPanel;

	public JFCTextPaneAppPanel(PDFGraphicsDemo pdfGraphicsDemo)
	{
		this.pdfGraphicsDemo = pdfGraphicsDemo;
		setLayout(new BorderLayout(5, 5));

		textPane = new JTextPane();
		reset();

		rightAppPanel = new RightAppPanel(pdfGraphicsDemo, textPane);

		leftPanel = new JPanel();
		leftPanel.setLayout(new BorderLayout(5, 5));
		leftPanel.add(new JScrollPane(textPane), BorderLayout.CENTER);

		add(leftPanel, BorderLayout.CENTER);
		add(rightAppPanel, BorderLayout.EAST);
		add(new JLabel("Note : Select the JTextPane's Text to apply the TextAttributes", SwingConstants.LEFT), BorderLayout.SOUTH);
	}

	public JTextPane getTextPane()
	{
		return textPane;
	}

	public void reset()
	{
		textPane.setText("This is a JFC TextPane\nTo view page setup click PageSetup\nTo view preview click Preview\nTo take print click Print");
		textPane.setFont(new Font("Dialog", Font.PLAIN, 12));
		textPane.setForeground(Color.black);
	}
}

class JFCTableAppPanel extends JPanel
{
	PDFGraphicsDemo pdfGraphicsDemo;

	JTable table;

	JPanel leftPanel;
	RightAppPanel rightAppPanel;

	public JFCTableAppPanel(PDFGraphicsDemo pdfGraphicsDemo)
	{
		this.pdfGraphicsDemo = pdfGraphicsDemo;
		setLayout(new BorderLayout(5, 5));

		DefaultTableModel model = new DefaultTableModel();
		table = new JTable(model);
		
		table.setAutoResizeMode(javax.swing.JTable.AUTO_RESIZE_OFF);
		reset();

		rightAppPanel = new RightAppPanel(pdfGraphicsDemo, table);

		leftPanel = new JPanel();
		leftPanel.setLayout(new BorderLayout(5, 5));
		leftPanel.add(new JScrollPane(table), BorderLayout.CENTER);

		add(leftPanel, BorderLayout.CENTER);
		add(rightAppPanel, BorderLayout.EAST);
	}

	public JTable getTable()
	{
		return table;
	}

	public void reset()
	{
		Vector columnNames = new Vector();
		columnNames.addElement("Employee ID");
		columnNames.addElement("First Name");
		columnNames.addElement("Last Name");
		columnNames.addElement("Address");
		columnNames.addElement("Salary");
		
		Vector data = createData();

		DefaultTableModel model = (DefaultTableModel)table.getModel();
		model.setDataVector(data, columnNames);
		
		table.getColumn(table.getColumnName(0)).setPreferredWidth(105);
		table.getColumn(table.getColumnName(1)).setPreferredWidth(80);
		table.getColumn(table.getColumnName(2)).setPreferredWidth(80);
		table.getColumn(table.getColumnName(3)).setPreferredWidth(304);
		table.getColumn(table.getColumnName(4)).setPreferredWidth(90);
	}
	
	private Vector createData()
	{
		Vector data = new Vector();
		Vector rowData = new Vector();

		rowData = new Vector();
		rowData.addElement("emp-00000" + data.size() + 1);
		rowData.addElement("Fran");
		rowData.addElement("Whitney");
		rowData.addElement("49 East Washington Street-Needham-MA-Zip 02192");
		rowData.addElement("$45700.000");
		data.addElement(rowData);

		rowData = new Vector();
		rowData.addElement("emp-00000" + data.size() + 1);
		rowData.addElement("Matthew");
		rowData.addElement("Cobb");
		rowData.addElement("77 Pleasant Street-Waltham-MA-Zip 02154");
		rowData.addElement("$62000.000");
		data.addElement(rowData);

		rowData = new Vector();
		rowData.addElement("emp-00000" + data.size() + 1);
		rowData.addElement("Philip");
		rowData.addElement("Chin");
		rowData.addElement("59 Pond Street-Atlanta-GA-Zip 30339");
		rowData.addElement("$38500.000");
		data.addElement(rowData);

		rowData = new Vector();
		rowData.addElement("emp-00000" + data.size() + 1);
		rowData.addElement("Julie");
		rowData.addElement("Jordan");
		rowData.addElement("144 Great Plain Avenue-Winchester-MA-Zip 01890");
		rowData.addElement("$51432.000");
		data.addElement(rowData);

		rowData = new Vector();
		rowData.addElement("emp-00000" + data.size() + 1);
		rowData.addElement("Robert");
		rowData.addElement("Breault");
		rowData.addElement("58 Cherry Street-Milton-MA-Zip 02186");
		rowData.addElement("$57490.000");
		data.addElement(rowData);

		rowData = new Vector();
		rowData.addElement("emp-00000" + data.size() + 1);
		rowData.addElement("Melissa");
		rowData.addElement("Espinoza");
		rowData.addElement("112 Apple Tree Way-Stow-MA-Zip 01775");
		rowData.addElement("$36490.000");
		data.addElement(rowData);

		rowData = new Vector();
		rowData.addElement("emp-00000" + data.size() + 1);
		rowData.addElement("Jeannett");
		rowData.addElement("Bertrand");
		rowData.addElement("209 Concord Street-Acton-MA-Zip 01720");
		rowData.addElement("$32780.000");
		data.addElement(rowData);

		rowData = new Vector();
		rowData.addElement("emp-00000" + data.size() + 1);
		rowData.addElement("Marc");
		rowData.addElement("Dill");
		rowData.addElement("89 Hancock Street-Milton-MA-Zip 02186");
		rowData.addElement("$54800.000");
		data.addElement(rowData);

		rowData = new Vector();
		rowData.addElement("emp-00000" + data.size() + 1);
		rowData.addElement("Jane");
		rowData.addElement("Francis");
		rowData.addElement("12 Hawthorne Drive-Concord-MA-Zip 01742");
		rowData.addElement("$53870.000");
		data.addElement(rowData);

		rowData = new Vector();
		rowData.addElement("emp-00000" + data.size() + 1);
		rowData.addElement("Natasha");
		rowData.addElement("Shishov");
		rowData.addElement("15 Milk Street-Waltham-MA-Zip 02154");
		rowData.addElement("$72995.000");
		data.addElement(rowData);

		rowData = new Vector();
		rowData.addElement("emp-00000" + data.size() + 1);
		rowData.addElement("Kurt");
		rowData.addElement("Driscoll");
		rowData.addElement("154 School Street-Waltham-MA-Zip 02154");
		rowData.addElement("$48023.690");
		data.addElement(rowData);

		rowData = new Vector();
		rowData.addElement("emp-00000" + data.size() + 1);
		rowData.addElement("Rodrigo");
		rowData.addElement("Guevara");
		rowData.addElement("East Main Street-Framingham-MA-Zip 01701");
		rowData.addElement("$42998.000");
		data.addElement(rowData);

		rowData = new Vector();
		rowData.addElement("emp-00000" + data.size() + 1);
		rowData.addElement("Ram");
		rowData.addElement("Gowda");
		rowData.addElement("79 Page Street-Natick-MA-Zip 01760");
		rowData.addElement("$59840.000");
		data.addElement(rowData);
		
		rowData = new Vector();
		rowData.addElement("emp-00000" + data.size() + 1);
		rowData.addElement("Fran");
		rowData.addElement("Whitney");
		rowData.addElement("49 East Washington Street-Needham-MA-Zip 02192");
		rowData.addElement("$45700.000");
		data.addElement(rowData);

		rowData = new Vector();
		rowData.addElement("emp-00000" + data.size() + 1);
		rowData.addElement("Matthew");
		rowData.addElement("Cobb");
		rowData.addElement("77 Pleasant Street-Waltham-MA-Zip 02154");
		rowData.addElement("$62000.000");
		data.addElement(rowData);

		rowData = new Vector();
		rowData.addElement("emp-00000" + data.size() + 1);
		rowData.addElement("Philip");
		rowData.addElement("Chin");
		rowData.addElement("59 Pond Street-Atlanta-GA-Zip 30339");
		rowData.addElement("$38500.000");
		data.addElement(rowData);

		rowData = new Vector();
		rowData.addElement("emp-00000" + data.size() + 1);
		rowData.addElement("Julie");
		rowData.addElement("Jordan");
		rowData.addElement("144 Great Plain Avenue-Winchester-MA-Zip 01890");
		rowData.addElement("$51432.000");
		data.addElement(rowData);

		rowData = new Vector();
		rowData.addElement("emp-00000" + data.size() + 1);
		rowData.addElement("Robert");
		rowData.addElement("Breault");
		rowData.addElement("58 Cherry Street-Milton-MA-Zip 02186");
		rowData.addElement("$57490.000");
		data.addElement(rowData);

		rowData = new Vector();
		rowData.addElement("emp-00000" + data.size() + 1);
		rowData.addElement("Melissa");
		rowData.addElement("Espinoza");
		rowData.addElement("112 Apple Tree Way-Stow-MA-Zip 01775");
		rowData.addElement("$36490.000");
		data.addElement(rowData);

		rowData = new Vector();
		rowData.addElement("emp-00000" + data.size() + 1);
		rowData.addElement("Jeannett");
		rowData.addElement("Bertrand");
		rowData.addElement("209 Concord Street-Acton-MA-Zip 01720");
		rowData.addElement("$32780.000");
		data.addElement(rowData);

		rowData = new Vector();
		rowData.addElement("emp-00000" + data.size() + 1);
		rowData.addElement("Marc");
		rowData.addElement("Dill");
		rowData.addElement("89 Hancock Street-Milton-MA-Zip 02186");
		rowData.addElement("$54800.000");
		data.addElement(rowData);

		rowData = new Vector();
		rowData.addElement("emp-00000" + data.size() + 1);
		rowData.addElement("Jane");
		rowData.addElement("Francis");
		rowData.addElement("12 Hawthorne Drive-Concord-MA-Zip 01742");
		rowData.addElement("$53870.000");
		data.addElement(rowData);

		rowData = new Vector();
		rowData.addElement("emp-00000" + data.size() + 1);
		rowData.addElement("Natasha");
		rowData.addElement("Shishov");
		rowData.addElement("15 Milk Street-Waltham-MA-Zip 02154");
		rowData.addElement("$72995.000");
		data.addElement(rowData);

		rowData = new Vector();
		rowData.addElement("emp-00000" + data.size() + 1);
		rowData.addElement("Kurt");
		rowData.addElement("Driscoll");
		rowData.addElement("154 School Street-Waltham-MA-Zip 02154");
		rowData.addElement("$48023.690");
		data.addElement(rowData);

		rowData = new Vector();
		rowData.addElement("emp-00000" + data.size() + 1);
		rowData.addElement("Rodrigo");
		rowData.addElement("Guevara");
		rowData.addElement("East Main Street-Framingham-MA-Zip 01701");
		rowData.addElement("$42998.000");
		data.addElement(rowData);

		rowData = new Vector();
		rowData.addElement("emp-00000" + data.size() + 1);
		rowData.addElement("Ram");
		rowData.addElement("Gowda");
		rowData.addElement("79 Page Street-Natick-MA-Zip 01760");
		rowData.addElement("$59840.000");
		data.addElement(rowData);

		rowData = new Vector();
		rowData.addElement("emp-00000" + data.size() + 1);
		rowData.addElement("Fran");
		rowData.addElement("Whitney");
		rowData.addElement("49 East Washington Street-Needham-MA-Zip 02192");
		rowData.addElement("$45700.000");
		data.addElement(rowData);

		rowData = new Vector();
		rowData.addElement("emp-00000" + data.size() + 1);
		rowData.addElement("Matthew");
		rowData.addElement("Cobb");
		rowData.addElement("77 Pleasant Street-Waltham-MA-Zip 02154");
		rowData.addElement("$62000.000");
		data.addElement(rowData);

		rowData = new Vector();
		rowData.addElement("emp-00000" + data.size() + 1);
		rowData.addElement("Philip");
		rowData.addElement("Chin");
		rowData.addElement("59 Pond Street-Atlanta-GA-Zip 30339");
		rowData.addElement("$38500.000");
		data.addElement(rowData);

		rowData = new Vector();
		rowData.addElement("emp-00000" + data.size() + 1);
		rowData.addElement("Julie");
		rowData.addElement("Jordan");
		rowData.addElement("144 Great Plain Avenue-Winchester-MA-Zip 01890");
		rowData.addElement("$51432.000");
		data.addElement(rowData);

		rowData = new Vector();
		rowData.addElement("emp-00000" + data.size() + 1);
		rowData.addElement("Robert");
		rowData.addElement("Breault");
		rowData.addElement("58 Cherry Street-Milton-MA-Zip 02186");
		rowData.addElement("$57490.000");
		data.addElement(rowData);

		rowData = new Vector();
		rowData.addElement("emp-00000" + data.size() + 1);
		rowData.addElement("Melissa");
		rowData.addElement("Espinoza");
		rowData.addElement("112 Apple Tree Way-Stow-MA-Zip 01775");
		rowData.addElement("$36490.000");
		data.addElement(rowData);

		rowData = new Vector();
		rowData.addElement("emp-00000" + data.size() + 1);
		rowData.addElement("Jeannett");
		rowData.addElement("Bertrand");
		rowData.addElement("209 Concord Street-Acton-MA-Zip 01720");
		rowData.addElement("$32780.000");
		data.addElement(rowData);

		rowData = new Vector();
		rowData.addElement("emp-00000" + data.size() + 1);
		rowData.addElement("Marc");
		rowData.addElement("Dill");
		rowData.addElement("89 Hancock Street-Milton-MA-Zip 02186");
		rowData.addElement("$54800.000");
		data.addElement(rowData);

		rowData = new Vector();
		rowData.addElement("emp-00000" + data.size() + 1);
		rowData.addElement("Jane");
		rowData.addElement("Francis");
		rowData.addElement("12 Hawthorne Drive-Concord-MA-Zip 01742");
		rowData.addElement("$53870.000");
		data.addElement(rowData);

		rowData = new Vector();
		rowData.addElement("emp-00000" + data.size() + 1);
		rowData.addElement("Natasha");
		rowData.addElement("Shishov");
		rowData.addElement("15 Milk Street-Waltham-MA-Zip 02154");
		rowData.addElement("$72995.000");
		data.addElement(rowData);

		rowData = new Vector();
		rowData.addElement("emp-00000" + data.size() + 1);
		rowData.addElement("Kurt");
		rowData.addElement("Driscoll");
		rowData.addElement("154 School Street-Waltham-MA-Zip 02154");
		rowData.addElement("$48023.690");
		data.addElement(rowData);

		rowData = new Vector();
		rowData.addElement("emp-00000" + data.size() + 1);
		rowData.addElement("Rodrigo");
		rowData.addElement("Guevara");
		rowData.addElement("East Main Street-Framingham-MA-Zip 01701");
		rowData.addElement("$42998.000");
		data.addElement(rowData);

		rowData = new Vector();
		rowData.addElement("emp-00000" + data.size() + 1);
		rowData.addElement("Ram");
		rowData.addElement("Gowda");
		rowData.addElement("79 Page Street-Natick-MA-Zip 01760");
		rowData.addElement("$59840.000");
		data.addElement(rowData);				
		
		return data;
	}	
}

class ButtonsAppPanel extends JPanel implements ActionListener
{
	PDFGraphicsDemo pdfGraphicsDemo;

	JButton pdfButton;
	JButton resetButton;

	public ButtonsAppPanel(PDFGraphicsDemo pdfGraphicsDemo)
	{
		this.pdfGraphicsDemo = pdfGraphicsDemo;
		setLayout(new FlowLayout(FlowLayout.CENTER, 5, 5));

		pdfButton = new JButton("Generate PDF");
		pdfButton.setMnemonic('G');
		pdfButton.addActionListener(this);

		resetButton = new JButton("Reset");
		resetButton.setMnemonic('R');
		resetButton.addActionListener(this);

		add(pdfButton);
		add(resetButton);
	}

	public void actionPerformed(ActionEvent e)
	{
		if(e.getSource() == pdfButton)
		{
			int index = pdfGraphicsDemo.tabPane.getSelectedIndex();

			Component comp = null;
			Component header = null;
			
			if(index == 0)
				comp = pdfGraphicsDemo.jfcTextAreaAppPanel.getTextArea();
			else if(index == 1)
				comp = pdfGraphicsDemo.jfcTextPaneAppPanel.getTextPane();
			else if(index == 2)
			{
				comp = pdfGraphicsDemo.jfcTableAppPanel.getTable();
				header = ((JTable)comp).getTableHeader();
			}
			else if(index == 3)
				comp = pdfGraphicsDemo.jfcPanelAppPanel.getPanel();
			
			if(comp != null)
			{
				int x = 0;
				int y = 0;
				int width = comp.getSize().width;
				int height = comp.getSize().height;
				
				if(header != null)
					height += header.getSize().height;
				
				int pageWidth = width;
				int pageHeight = (int)MediaBox.MEDIA_a4.getHeight();
				
				PDF pdf = new PDF();
				MediaBox mediaBox = new MediaBox(-10, 10, pageWidth + 20, pageHeight + 20);
				PDFGraphics pg = new PDFGraphics(pdf, mediaBox);
					
				int noOfPages = (int)Math.ceil(height * 1.0 / pageHeight);
				
				for(int i=0; i<noOfPages; i++)
				{
					y = i * pageHeight;
					
					pg.createPage();
					
					if(header != null && i == 0)
					{
						paintComponent(pg, header, x, y);
						pg.translate(0, header.getSize().height);
					}
					paintComponent(pg, comp, x, y);			
				}
				
				try {
					FileOutputStream out = new FileOutputStream("test.pdf");
					pdf.write(out);
					out.close();
					out = null;
				} catch(Exception ex) {
					ex.printStackTrace();
				}
								
				pg.dispose();
				pg = null;
			}
		}
		else if(e.getSource() == resetButton)
		{
			pdfGraphicsDemo.jfcTextAreaAppPanel.reset();
			pdfGraphicsDemo.jfcTextPaneAppPanel.reset();
			pdfGraphicsDemo.jfcTableAppPanel.reset();
			pdfGraphicsDemo.jfcPanelAppPanel.reset();
		}
	}
	
	private void paintComponent(PDFGraphics g, Component c, int x, int y)
	{
		boolean wasDoubleBuffered = false;
		if((c instanceof JComponent) && ((JComponent)c).isDoubleBuffered()) 
		{
	    wasDoubleBuffered = true;
	    ((JComponent)c).setDoubleBuffered(false);
		}

		g.translate(-x, -y);		
		c.printAll(g);
		g.translate(x, y);		

		if(wasDoubleBuffered && (c instanceof JComponent))
	    ((JComponent)c).setDoubleBuffered(true);
	}
}

class RightAppPanel extends JPanel implements ActionListener
{
	PDFGraphicsDemo pdfGraphicsDemo;
	Component component;
	String componentInstance = "";

	JLabel fontLabel;
	JLabel foregroundLabel;
	JButton fontButton;
	JButton foregroundButton;
	JLabel textAttributesLabel;
	JButton textAttributesButton;

	JButton addRowButton;
	JButton addColumnButton;

	String editorStatus = "";

	JPanel upperPanel;

	public RightAppPanel(PDFGraphicsDemo pdfGraphicsDemo, Component component)
	{
		this.pdfGraphicsDemo = pdfGraphicsDemo;
		this.component = component;

		if(component instanceof JTextArea)	componentInstance = "jtextarea";
		if(component instanceof JTextPane)	componentInstance = "jtextpane";
		if(component instanceof JTable)	componentInstance = "jtable";

		setLayout(new BorderLayout(0, 0));
		upperPanel = new JPanel();

		if(componentInstance.equals("jtextarea") || componentInstance.equals("jtextpane"))
		{
			upperPanel.setLayout(new GridLayout(3, 2, 10, 5));

			fontLabel = new JLabel("Font");
			foregroundLabel = new JLabel("Foreground");

			fontButton = new JButton("Click to Edit");
			fontButton.addActionListener(this);
			
			foregroundButton = new JButton("Click To Edit");
			foregroundButton.addActionListener(this);

			upperPanel.add(fontLabel);				upperPanel.add(fontButton);
			upperPanel.add(foregroundLabel);	upperPanel.add(foregroundButton);

			if(componentInstance.equals("jtextpane"))
			{
				upperPanel.setLayout(new GridLayout(5, 2, 10, 5));
				textAttributesLabel = new JLabel("Text Attributes");
				textAttributesButton = new JButton("Click To Edit");
				textAttributesButton.addActionListener(this);
				upperPanel.add(textAttributesLabel);
				upperPanel.add(textAttributesButton);
			}
		}
		else if(componentInstance.equals("jtable"))
		{
			upperPanel.setLayout(new GridLayout(2, 1, 10, 5));
			addRowButton = new JButton("Add Row");
			addRowButton.addActionListener(this);
			addColumnButton = new JButton("Add Column");
			addColumnButton.addActionListener(this);

			upperPanel.add(addRowButton);
			upperPanel.add(addColumnButton);
		}

		add(upperPanel, BorderLayout.NORTH);
		add(new JPanel(), BorderLayout.CENTER);
	}

	public void actionPerformed(ActionEvent e)
	{
		if(e.getSource() == addRowButton)
		{
			Vector vector = new Vector();
			DefaultTableModel model = (DefaultTableModel)(((JTable)component).getModel());
			model.addRow(vector);
		}
		else if(e.getSource() == addColumnButton)
		{
			Vector vector = new Vector();
			DefaultTableModel model = (DefaultTableModel)(((JTable)component).getModel());
			model.addColumn("Column", vector);
		}
		else if(e.getSource() == fontButton)
		{
			ExtJDialog extJDialog = new ExtJDialog(pdfGraphicsDemo, "Font Editor", pdfGraphicsDemo.fontEditor.getCustomEditor());
			if(extJDialog.OK_PRESSED)
			{
				Font fontValue = (Font)pdfGraphicsDemo.fontEditor.getValue();
				if(component instanceof TextArea)
					((TextArea)component).setFont((Font)pdfGraphicsDemo.fontEditor.getValue());
				else if(component instanceof JTextArea)
					((JTextArea)component).setFont((Font)pdfGraphicsDemo.fontEditor.getValue());
				else if(component instanceof JTextPane)
					((JTextPane)component).setFont((Font)pdfGraphicsDemo.fontEditor.getValue());
			}
		}
		else if(e.getSource() == foregroundButton)
		{
			pdfGraphicsDemo.colorEditor.setValue(foregroundButton.getBackground());
			ExtJDialog extJDialog = new ExtJDialog(pdfGraphicsDemo, "Color Editor", pdfGraphicsDemo.colorEditor.getCustomEditor());
			if(extJDialog.OK_PRESSED)
			{			
				foregroundButton.setBackground((Color)pdfGraphicsDemo.colorEditor.getValue());
				foregroundButton.setForeground(new Color(255 - foregroundButton.getBackground().getRed(), 255 - foregroundButton.getBackground().getGreen(), 255 - foregroundButton.getBackground().getBlue()));
	
				if(component instanceof TextArea)
					((TextArea)component).setForeground((Color)pdfGraphicsDemo.colorEditor.getValue());
				else if(component instanceof JTextArea)
					((JTextArea)component).setForeground((Color)pdfGraphicsDemo.colorEditor.getValue());
				else if(component instanceof JTextPane)
					((JTextPane)component).setForeground((Color)pdfGraphicsDemo.colorEditor.getValue());
			}
		}
		else if(e.getSource() == textAttributesButton)
		{
			JTextAttributesDialog dialog = new JTextAttributesDialog((JTextPane)component, new JFrame());
		}
	}
}